﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using FTD2XX_NET;

namespace MP_Builder_Sample
{
    public partial class Form1 : Form
    {
        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// クラス／テーブル
        /// Class / Table
        /// </summary>
        //
        public string VerDate = "Ver.2023.06.02";
        //
        public clsFTDI hFTDI = new clsFTDI();
        public FlowProcess hFlow = new FlowProcess();
        public DataSample hDataSmp = new DataSample();
        public FlowSample hFlowSmp = new FlowSample();
        //
        public bool LSI_stop;

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// 
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Form作成
        /// Form Creation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            if (hFTDI.LSI_Open(this, hFlowSmp.Description) != FTDI.FT_STATUS.FT_OK)
            {
                //オープン失敗
                //Open Failure
                MessageBox.Show("  \" " + hFlowSmp.Description + " \"   is not found.", "Error !!!");
                Close();
            }
            else
            {
                //受信バッファをクリア
                hFTDI.LSI_CheckRead(ref hFTDI.ReadBuffer);

                hFTDI.Ds = hDataSmp.Ds;
                hFlow.InitPulseGenelator(this, 0);

                //
                this.SampleExecute.Enabled = true;
                this.bt_stop.Enabled = false;
                this.bt_exit.Enabled = true;

                //
                this.l_target.Text = hFlowSmp.Targetname;

                //画面中央に配置
                //Placed in the center of the screen
                this.Left = (System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width - this.Width) / 2;
                this.Top = (System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height - this.Height) / 2;

                //
                this.Text = "Sample   " + VerDate;
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Form破棄
        /// Discard Form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //全軸停止までクローズさせない
            hFlow.WaitMotionEnd(this, 0, hDataSmp.UseAxis);
            //
            if(hFTDI.FlowData_on == false)
            {
                e.Cancel = true;
            }
            else
            {
                hFTDI.LSI_Close();
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// サンプルの実行
        /// Execution Sample
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SampleExecute_Click(object sender, EventArgs e)
        {
            //
            this.SampleExecute.Enabled = false;
            this.bt_stop.Enabled = true;
            this.bt_exit.Enabled = false;
            //
            hFTDI.FlowData_on = true;
            hFTDI.FlowData_stop = false;//中断フラグ＝オフ
            this.Focus();
            //メイン側に処理が移行すると、Form1側のボタンが1回無視される。これを防ぐためフォーカスを保持させる。
            //When processing shifts to the main side, the button on Form 1 side is ignored once. In order to prevent this, keep focus.
            if (hFlowSmp.FlowChartExe(this) == 0)
            {
                //全ての動作が完了
                MessageBox.Show("Sample operation end.", "message");
            }
            else
            {
                //中断
                MessageBox.Show("Sample operation interrupted.", "message");
            }
            //ソフト終了許可
            hFTDI.FlowData_stop = true;
            //
            this.SampleExecute.Enabled = true;
            this.bt_stop.Enabled = false;
            this.bt_exit.Enabled = true;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// サンプルの停止
        /// Stop Sample
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Bt_stop_Click(object sender, EventArgs e)
        {
            //
            hFTDI.FlowData_stop = true;//中断フラグ＝オン // Interrupt flag = on
            hFTDI.FlowData_on = false;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// ソフトウェアの終了
        /// End of software
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Bt_exit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
