﻿//////////////////////////////////////////////////////////////////////////////
//
//  FTDI access source  ( for PCD2112 )
//
//  NIPPON PULSE MOTOR Co., Ltd.
//
//////////////////////////////////////////////////////////////////////////////

using System;
using System.Runtime.InteropServices;
using FTD2XX_NET;
using System.Threading;
using System.Diagnostics;
using System.Linq;

namespace MP_Builder_Sample
{
    public partial class clsFTDI
    {
        ////////////////////////////////////////////////////////////////////
        /// 
        public FTDI ftHandle = new FTDI();
        //
        public byte Ds = 0;//Device number is changed in software
        //keep status
        public uint[] MSTS = new uint[4];
        public uint[] SSTS = new uint[4];
        //
        //Maximum definition
        private const int MAX_TAG = 65536;
        private const int MAX_REG = 256;
        private const int MAX_BUFF = 4096;
        //
        public int index = 0;
        public byte[] WriteBuffer = new byte[MAX_BUFF];
        public int indexS = 0;//for status read
        public byte[] WriteBufferS = new byte[MAX_BUFF];//for status read
        public byte[] ReadBuffer = new byte[MAX_BUFF];
        public bool FlowData_on = true;//active flag
        public bool FlowData_stop = false;//Suspend flag = off

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// use as union
        /// </summary>
        [StructLayout(LayoutKind.Explicit)]
        public struct TBytes
        {
            [FieldOffset(0)]
            public byte Byte0;

            [FieldOffset(1)]
            public byte Byte1;

            [FieldOffset(2)]
            public byte Byte2;

            [FieldOffset(3)]
            public byte Byte3;

            [FieldOffset(0)]
            public uint Word;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// FTDI connection processing
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS LSI_Open(Form1 fm1, string description)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            ftStatus = ftHandle.OpenByDescription(description);

            if (!ftHandle.IsOpen)
            {   // error
                Debug.WriteLine("Error: FTDI.penByDescription! [ " + ftStatus.ToString() + " ]");
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.ResetDevice
            ftStatus = ftHandle.ResetDevice();

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.ResetDevice! [ " + ftStatus.ToString() + " ]");
                ftHandle.Close();
                return ftStatus;
            }

            Thread.Sleep(100);     //Wair fot reset process

            ////////////////////////////////////////////////////////////////////
            // FTDI.SetLatency (1ms)
            ftStatus = ftHandle.SetLatency(1);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.SetLatency! [ " + ftStatus.ToString() + " ]");
                ftHandle.Close();
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.SetTimeouts
            uint ReadTimeout = 1000;    // 1000ms
            uint WriteTimeout = 1000;   // 1000ms

            ftStatus = ftHandle.SetTimeouts(ReadTimeout, WriteTimeout);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.SetTimeouts! [ " + ftStatus.ToString() + " ]");
                ftHandle.Close();
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.SetBitMode (MPSSE Mode)
            byte Mask = 0;
            byte BitMode = FTDI.FT_BIT_MODES.FT_BIT_MODE_MPSSE;

            ftStatus = ftHandle.SetBitMode(Mask, BitMode);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.SetBitMode! [ " + ftStatus.ToString() + " ]");
                ftHandle.Close();
                return ftStatus;
            }
            //
            ////////////////////////////////////////////////////////////////////
            // FTDI.Write (SPI=10MHz)
            byte[] dataBuffer = new byte[] { 0x8D, // Disable 3 Phase Data Clocking (Waveforms other than I2C)
                                             0x86, 0x02, 0x00 };    // SPI=10MHz
            int numBytesToWrite = 4;
            uint numBytesWritten = 0;

            ftStatus = ftHandle.Write(dataBuffer, numBytesToWrite, ref numBytesWritten);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.Write! [ " + ftStatus.ToString() + " ]");
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.Write (Set initial port value)
            dataBuffer[0] = 0x80;
            dataBuffer[1] = 0x18;//RST=High, CS=High
            dataBuffer[2] = 0x1B;
            numBytesToWrite = 3;

            ftStatus = ftHandle.Write(dataBuffer, numBytesToWrite, ref numBytesWritten);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.Write! [ " + ftStatus.ToString() + " ]");
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.Write (12MHz -> 60MHz)  SPI is 10MHz
            dataBuffer[0] = 0x8A;// 60MHz
            numBytesToWrite = 1;
            numBytesWritten = 0;

            ftStatus = ftHandle.Write(dataBuffer, numBytesToWrite, ref numBytesWritten);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.Write! [ " + ftStatus.ToString() + " ]");
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            // FTDI.Purge
            //  Clear transmit and receive buffers
            ftStatus = ftHandle.Purge(FTDI.FT_PURGE.FT_PURGE_RX | FTDI.FT_PURGE.FT_PURGE_TX);

            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.Purge! [ " + ftStatus.ToString() + " ]");
                ftHandle.Close();
                return ftStatus;
            }

            ////////////////////////////////////////////////////////////////////
            return ftStatus;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public FTDI.FT_STATUS LSI_Close()
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            ////////////////////////////////////////////////////////////////////
            // FTDI.Close
            ftStatus = ftHandle.Close();

            if (ftHandle.IsOpen)
            {   // error
                Debug.WriteLine("Error: FTDI.Close! [" + ftStatus.ToString() + "]");
                return ftStatus;
            }

            return ftStatus;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// FTDI Write
        /// </summary>
        /// <param name="dataBuffer"></param>
        /// <param name="numBytesToWrite"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS LSI_Write(byte[] dataBuffer, int numBytesToWrite)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            ////////////////////////////////////////////////////////////////////
            // FTDI.Write
            uint numBytesWritten = 0;

            ftStatus = ftHandle.Write(dataBuffer, numBytesToWrite, ref numBytesWritten);

            ////////////////////////////////////////////////////////////////////
            // Error Check
            if (ftStatus != FTDI.FT_STATUS.FT_OK)
            {   // error
                Debug.WriteLine("Error: FTDI.Write! [ " + ftStatus.ToString() + " ]");
                return ftStatus;
            }

            if (numBytesToWrite != numBytesWritten)
            {   // error
                Debug.WriteLine(
                    "Error: FTDI.Write! [ numBytesToWrite: " + numBytesToWrite.ToString() +
                    " /= numBytesWritten: " + numBytesWritten.ToString() + " ]");
                return FTDI.FT_STATUS.FT_OTHER_ERROR;
            }

            ////////////////////////////////////////////////////////////////////
            return ftStatus;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Garbage may remain after forced termination.
        /// This function eliminates garbage.
        /// </summary>
        /// <param name="dataBuffer"></param>
        /// <returns></returns>
        public uint LSI_CheckRead(ref byte[] dataBuffer)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            uint RxQueue = 0;
            uint Dumy = 0;

            ftStatus = ftHandle.GetRxBytesAvailable(ref RxQueue);
            //
            if ((ftStatus == FTDI.FT_STATUS.FT_OK) && (RxQueue > 0))
            {
                ftStatus = ftHandle.Read(dataBuffer, RxQueue, ref Dumy);
                RxQueue = Dumy;
            }
            //
            return(RxQueue);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// FTDI Read(Hight speed)
        /// </summary>
        /// <param name="dataBuffer"></param>
        /// <param name="numBytesRead"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS LSI_Read_first(ref byte[] dataBuffer, ref uint numBytesRead)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            ////////////////////////////////////////////////////////////////////
            // FTDI.Read

            int bad = 0;
            uint RxQueue = 0;
            do
            {
                ftStatus = ftHandle.GetRxBytesAvailable(ref RxQueue);
                bad++;
                if (bad > 5000)
                {
                    ftStatus = FTDI.FT_STATUS.FT_EEPROM_READ_FAILED;
                    break;
                }

            } while (RxQueue < numBytesRead);

            numBytesRead = RxQueue;
            uint numBytesToRead = 0;
            do
            {
                ftStatus = ftHandle.Read(dataBuffer, numBytesRead, ref numBytesToRead);

                if (ftStatus != FTDI.FT_STATUS.FT_OK)
                {   // error
                    Debug.WriteLine("Error: FTDI.Read! [" + ftStatus.ToString() + "]");
                    return ftStatus;
                }
                //
                numBytesRead = numBytesRead - numBytesToRead;
            }
            while (numBytesRead > 0);

            return ftStatus;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Set buffer the command Note that only one axis is processed
        ///   comm ： register write command
        ///   RegD ： write data
        /// </summary>
        /// <returns></returns>
        public void Write_REG(byte comm, uint RegD, int num)
        {
            ////////////////////////////////////////////////////////////////////
            var bytes = new TBytes();

            bytes.Word = RegD; //共用体へ代入

            //CS=Low
            WriteBuffer[index++] = 0x80;
            WriteBuffer[index++] = 0x00;
            WriteBuffer[index++] = 0x1B;//CS:ADBUS4
            //
            WriteBuffer[index++] = 0x11;//SPI(Clock Data Bytes Out on +ve Clock Edge MSB First (no Read))
            WriteBuffer[index++] = (byte)(num + 1);//Length L (num + 2)byte
            WriteBuffer[index++] = 0x00;//Length H
            WriteBuffer[index++] = (byte)(0x02 | (Ds << 4)); // Write // DS="00"
            WriteBuffer[index++] = comm;
            for(int loops = 0; loops < num; loops++)
            {
                int sft = (num - loops - 1) * 8;
                WriteBuffer[index++] = (byte)((RegD >> sft) & 0x000000FF);
            }
            //CS=High
            WriteBuffer[index++] = 0x80;
            WriteBuffer[index++] = 0x18;
            WriteBuffer[index++] = 0x1B;//CS:ADBUS4
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Set buffer the command Note that only one axis is processed
        ///   comm ： command
        /// </summary>
        /// <returns></returns>
        public void Write_COM(byte comm)
        {
            ////////////////////////////////////////////////////////////////////
            //CS=Low
            WriteBuffer[index++] = 0x80;
            WriteBuffer[index++] = 0x00;
            WriteBuffer[index++] = 0x1B;//CS:ADBUS4
            //
            WriteBuffer[index++] = 0x11;//SPI(Clock Data Bytes Out on +ve Clock Edge MSB First (no Read))
            WriteBuffer[index++] = 0x02;//Length L 3byte
            WriteBuffer[index++] = 0x00;//Length H
            WriteBuffer[index++] = (byte)(0x02 | (Ds << 4)); // Write // DS="00"
            WriteBuffer[index++] = (byte)(0x25);//PCD2112 command Address
            WriteBuffer[index++] = comm;
            //CS=High
            WriteBuffer[index++] = 0x80;
            WriteBuffer[index++] = 0x18;
            WriteBuffer[index++] = 0x1B;//CS:ADBUS4
       }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Send seted data via USB
        /// </summary>
        /// <returns></returns>
        public FTDI.FT_STATUS SendUsb(byte[] WriteBuffer, ref int index)
        {
            ////////////////////////////////////////////////////////////////////
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            //
            if (index > 0)
            {
                ftStatus = LSI_Write(WriteBuffer, index);
                index = 0;
            }
            //
            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Set buffer the status read commands
        /// </summary>
        public void Read_Status(int AxisConts)
        {
            //PCD2112 is only X-axis
            //
            //  status ------------------------------------------------
            //CS=Low
            WriteBufferS[indexS++] = 0x80;
            WriteBufferS[indexS++] = 0x00;
            WriteBufferS[indexS++] = 0x1B;//CS:ADBUS4
            //
            WriteBufferS[indexS++] = 0x35;//SPI(Out on negative edge, in on negative edge)
            WriteBufferS[indexS++] = 0x05;//Length L 6byte
            WriteBufferS[indexS++] = 0x00;//Length H
            WriteBufferS[indexS++] = (byte)(0x03 | (Ds << 4));// Read // DS="00"
            WriteBufferS[indexS++] = (byte)(0x26);//PCD2112 RSTS Address
            WriteBufferS[indexS++] = 0x00;//dumy
            WriteBufferS[indexS++] = 0x00;//dumy
            WriteBufferS[indexS++] = 0x00;//dumy
            WriteBufferS[indexS++] = 0x00;//dumy
            //CS=High
            WriteBufferS[indexS++] = 0x80;
            WriteBufferS[indexS++] = 0x18;
            WriteBufferS[indexS++] = 0x1B;//CS:ADBUS4
            //
            WriteBufferS[indexS++] = 0x87;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Pick up status and sub-status from USB read buffer "ReadBuffer"
        ///   ofset ： offset to the beginning of the data (not used in this source, just match the other arguments)
        ///   MSTS ： Main status read result
        /// </summary>
        /// <returns></returns>
        public void PickUpStatus(int ofset)
        {
            var bytes = new TBytes();

            //
            bytes.Byte3 = (byte)(ReadBuffer[ofset + 0]);//Data 31-24
            bytes.Byte2 = (byte)(ReadBuffer[ofset + 1]);//Data 23-16
            bytes.Byte1 = (byte)(ReadBuffer[ofset + 2]);//Data 15-8
            bytes.Byte0 = (byte)(ReadBuffer[ofset + 3]);//Data 7-0
            //
            MSTS[0] = (uint)(bytes.Word);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Wait for operation completion
        /// Do not move to the next operation until all of the specified axes are completed or the pre-registration is free
        ///
        ///  AxisConts : Number of LSI axes
        ///  UseAxis   : Specify axis to wait for status
        ///
        ///  return    : 0＝All specified axes are stopped
        ///            : 1＝All specified axes are in motion but pre-registration is empty or stopped
        ///            : 3＝One of the specified axes is in motion and pre-registration is not free
        /// </summary>
        public int WaitMsts(Form1 fm1, int AxisConts, int UseAxis)
        {
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            int[] cksts = new int[] {3, 3, 3, 3};
            //
            indexS = 0;
            Read_Status(AxisConts);
            //
            ftStatus = SendUsb(WriteBufferS, ref indexS);
            //
            if (ftStatus == FTDI.FT_STATUS.FT_OK)
            {
                uint numBytesRead = 6;
                ftStatus = LSI_Read_first(ref ReadBuffer, ref numBytesRead);
                //
                if (ftStatus == FTDI.FT_STATUS.FT_OK)
                {
                    PickUpStatus(2);
                }
                else
                {
                    //Forced termination because it could not be read
                    MSTS[0] = 0;
                }
            }
            //
            int loopA = 0;//1 axis only
            if ((MSTS[loopA] & 0x0F000000) == 0)
            {
                //stopped
                cksts[loopA] = 0;
            }
            //
            int ret = 0;
            if (ret < cksts[loopA])
            {
                ret = cksts[loopA];
            }
            return (ret);
        }

    } // class
}
